@@ -5,27 +5,34 @@ import logging
 import sys
 from os import path
 import argparse
-import inspect
 from pySRSLockin.driver import SRS_Lockin
 from pySRSLockin.plots import PlotObject
+from pySRSLockin.abstract_interface import abstract_interface, abstract_gui
 
 graphics_dir = path.join(path.dirname(__file__), 'graphics')
 
-class interface():
+class interface(abstract_interface):
+    """
+    Create a high-level interface with the device, and provide a GUI
+
+    ...
+
+    Attributes
+    ----------
+    output : dict
+        a dictionary containing the output data created by this interface
+    app : Qt.QApplication
+        The pyqt5 QApplication() object
+    mainwindow  : Qt.QWidget
+        The main window of the Qt application
+    instrument : SRS_Lockin
+        a driver object, for the low-level communication with the device
+    gui : gui()
+        instance of the class gui(), containing all guis element and basic function to set different states
+    """
     output = {'X':0,'Y':0,'mag':0,'theta':0} #We define this also as class variable, to make it possible to see which data is produced by this interface without having to create an object
 
-    def __init__(self, app, mainwindow, parent, name_logger=__package__):
-        # app           = The pyqt5 QApplication() object
-        # mainwindow    = Main Window of the application
-        # parent        = a QWidget (or QMainWindow) object that will be the parent for the gui of this device.
-        #                 When this script is ran on its own (i.e. not within another gui) mainwindow and parent are actually the same object
-        # name_logger   = The name of the logger used for this particular istance of the interface object. If none is specified, the name of package is used as logger name
-
-        self.mainwindow = mainwindow
-        self.app = app
-        self.parent = parent
-        self.mainwindow.child = self #make sure that the window embedding this interface knows about its child (this is mainly used to link the closing events when using multiple windows)
- 
+    def __init__(self, **kwargs):
         self.output = {'X':0,'Y':0,'mag':0,'theta':0} #We define this also as class variable, to make it possible to see which data is produced by this interface without having to create an object
         self.continuous_read = False #When this is set to True, the data from device are acquired continuosly at the rate set by self.refresh_time
         self.refresh_time = 0.2 #default refresh rate, in seconds
@@ -33,280 +40,40 @@ class interface():
         self.plot_window = None # QWidget object of the widget (i.e. floating window) that will contain the plot
         self.plot_object = None # PlotObject object of the plot where self.store_powers is plotted
         self.connected_device_name = ''
-        self._verbose = True #Keep track of whether this instance of the interface should produce logs or not
-        self._name_logger = ''
-
+        
         self.instrument = SRS_Lockin() 
-        self.name_logger = name_logger #Setting this property will also create the logger,set defaulat output style, and store the logger object in self.logger
+        super().__init__(**kwargs)
 
-    @property
-    def verbose(self):
-        return self.verbose
-
-    @verbose.setter
-    def verbose(self,verbose):
-        #When the verbose property of this interface is changed, we also update accordingly the level of the logger object
-        if verbose: loglevel = logging.INFO
-        else: loglevel = logging.CRITICAL
-        self.logger.setLevel(level=loglevel)
-
-    @property
-    def name_logger(self):
-        return self._name_logger
-
-    @name_logger.setter
-    def name_logger(self,name):
-        #Create logger, and set default output style.
-        self._name_logger = name
-        self.logger = logging.getLogger(self._name_logger)
-        self.verbose = self._verbose #This will automatically set the logger verbosity too
-        if not self.logger.handlers:
-            formatter = logging.Formatter(f"[{self.name_logger}]: %(message)s")
-            ch = logging.StreamHandler()
-            ch.setFormatter(formatter)
-            self.logger.addHandler(ch)
-        self.logger.propagate = False
- 
-    ### Functions to create GUI
-
-    def create_gui(self,plot=False): 
+    def create_gui(self,parent,plot=False): 
         '''
+            parent, QWidget        
+                a QWidget (or QMainWindow) object that will be the parent for the gui of this device.
+                When this script is ran on its own (i.e. not within another gui) self.mainwindow and parent are actually the same object
             plot, boolean
-            If set true, the GUI also generates a plot object (and a button to show/hide the plot) to plot the content of the self.stored_data object
-            NOTE: plot function still to be implemented
+                If set true, the GUI also generates a plot object (and a button to show/hide the plot) to plot the content of the self.stored_data object
         '''
-        self.widgets_enabled_when_connected = []     #The widgets in this list will only be enabled when the interface has succesfully connected to a powermeter
-        self.widgets_enabled_when_disconnected = []  #The widgets in this list will only be enabled when the interface is not connected to a powermeter
-        
-        hbox1 = Qt.QHBoxLayout()
-        self.label_DeviceList = Qt.QLabel("Devices: ")
-        self.combo_Devices = Qt.QComboBox()
-        #self.combo_Devices.setFixedWidth(300)
-        self.button_RefreshDeviceList = Qt.QPushButton("")
-        self.button_RefreshDeviceList.setIcon(QtGui.QIcon(path.join(graphics_dir,'refresh.png')))
-        self.button_RefreshDeviceList.clicked.connect(self.click_button_refresh_list_devices)
-        hbox1.addWidget(self.label_DeviceList)
-        hbox1.addWidget(self.combo_Devices,stretch=1)
-        hbox1.addWidget(self.button_RefreshDeviceList)
-        #hbox1.addStretch(1)
+        self.gui = gui(self,parent,plot)
+        self.gui.initialize() 
 
-        hbox2 = Qt.QHBoxLayout()
-        self.button_ConnectDevice = Qt.QPushButton("Connect")
-        self.button_ConnectDevice.clicked.connect(self.click_button_connect_disconnect)
-        self.button_SetAutoScale = Qt.QPushButton("Set Auto Scale")
-        self.button_SetAutoScale.clicked.connect(self.click_button_set_auto_scale)
-        self.label_TimeConstants = Qt.QLabel("Time constant (s): ")
-        self.combo_TimeConstants = Qt.QComboBox()
-        self.combo_TimeConstants.activated.connect(self.click_combo_time_constants)
+### Functions to talk with low-level driver
 
-        self.label_Sensitivities = Qt.QLabel("Sensitivity (V): ")
-        self.combo_Sensitivities = Qt.QComboBox()
-        self.combo_Sensitivities.activated.connect(self.click_combo_sensitivities)
-
-        hbox2.addWidget(self.button_ConnectDevice)
-        hbox2.addWidget(self.button_SetAutoScale)
-        hbox2.addWidget(self.label_TimeConstants)
-        hbox2.addWidget(self.combo_TimeConstants,stretch=1)
-        hbox2.addWidget(self.label_Sensitivities)
-        hbox2.addWidget(self.combo_Sensitivities,stretch=1)
-        #hbox2.addStretch(1)
-
-        hbox3 = Qt.QHBoxLayout()
-        hbox3_vbox1 = Qt.QVBoxLayout()
-        
-        self.button_StartPauseReading = Qt.QPushButton("")
-        self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'play.png')))
-        self.button_StartPauseReading.setToolTip('Start or pause the reading from the device. The previous data points are not discarded when pausing.') 
-        self.button_StartPauseReading.clicked.connect(self.click_button_StartPauseReading)
-        self.button_StopReading = Qt.QPushButton("")
-        self.button_StopReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'stop.png')))
-        self.button_StopReading.setToolTip('Stop the reading from the device. All previous data points are discarded.') 
-        self.button_StopReading.clicked.connect(self.click_button_StopReading)
-        hbox3_vbox1_hbox1 = Qt.QHBoxLayout()
-        hbox3_vbox1_hbox1.addWidget(self.button_StartPauseReading)  
-        hbox3_vbox1_hbox1.addWidget(self.button_StopReading)
-        
-        
-        self.label_RefreshTime = Qt.QLabel("Refresh time (s): ")
-        self.label_RefreshTime.setToolTip('Specifies how often data is read from the device (Minimum value = 0.001 s).') 
-        self.edit_RefreshTime  = Qt.QLineEdit()
-        self.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
-        self.edit_RefreshTime.setToolTip('Specifies how often data is read from the device (Minimum value = 0.001 s).') 
-        self.edit_RefreshTime.returnPressed.connect(self.press_enter_refresh_time)
-        self.edit_RefreshTime.setAlignment(QtCore.Qt.AlignRight)
-        self.edit_RefreshTime.setMaximumWidth(50)
-        hbox3_vbox1_hbox2 = Qt.QHBoxLayout()
-        hbox3_vbox1_hbox2.addWidget(self.label_RefreshTime)  
-        hbox3_vbox1_hbox2.addWidget(self.edit_RefreshTime,stretch=1)
-
-        hbox3_vbox1.addLayout(hbox3_vbox1_hbox1)
-        hbox3_vbox1.addLayout(hbox3_vbox1_hbox2)
-
-
-        fontLabel = QtGui.QFont("Times", 10,QtGui.QFont.Bold)
-        fontEdit = QtGui.QFont("Times", 10,QtGui.QFont.Bold)
-
-        self.label_Output = dict()    
-        self.edit_Output = dict() 
-        self.widget_Output = dict()     
-        for k in self.output.keys():		
-            self.label_Output[k] = Qt.QLabel(f"{k}: ")
-            self.label_Output[k].setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignCenter)
-            self.label_Output[k].setFont(fontLabel)
-            self.edit_Output[k] = Qt.QLineEdit()
-            self.edit_Output[k].setFont(fontEdit)
-            self.edit_Output[k].setMinimumWidth(60)
-            self.edit_Output[k].setAlignment(QtCore.Qt.AlignRight)
-            self.edit_Output[k].setReadOnly(True)
-            hbox_temp = Qt.QHBoxLayout()
-            hbox_temp.addWidget(self.label_Output[k])
-            hbox_temp.addWidget(self.edit_Output[k],stretch=1)
-            self.widget_Output[k] = Qt.QWidget()
-            self.widget_Output[k].setLayout(hbox_temp)
-
-        self.layout_ContainerOutputs = Qt.QGridLayout()
-        self.layout_ContainerOutputs.addWidget(self.widget_Output['X'], 0, 0)
-        self.layout_ContainerOutputs.addWidget(self.widget_Output['Y'], 1, 0)
-        self.layout_ContainerOutputs.addWidget(self.widget_Output['mag'], 0, 1)
-        self.layout_ContainerOutputs.addWidget(self.widget_Output['theta'], 1, 1)
-        self.layout_ContainerOutputs.setSpacing(0)
-        self.layout_ContainerOutputs.setContentsMargins(0, 0, 0, 0)
-        self.widget_ContainerOutputs = Qt.QWidget()
-        self.widget_ContainerOutputs.setLayout(self.layout_ContainerOutputs)
-        #self.button_ShowHidePlot = Qt.QPushButton("Show/Hide Plot")
-        #self.button_StopReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'stop.png')))
-        #self.button_ShowHidePlot.setToolTip('Show/Hide Plot.') 
-        #self.button_ShowHidePlot.clicked.connect(self.click_button_ShowHidePlot)
-
-        hbox3.addLayout(hbox3_vbox1)
-        #hbox3.addStretch(1)
-        hbox3.addWidget(self.widget_ContainerOutputs,stretch=1)
-        #hbox3.addWidget(self.button_ShowHidePlot)
-        #hbox3.addStretch(1)
-        #hbox3.setAlignment(QtCore.Qt.AlignTop)
-                
-        vbox = Qt.QVBoxLayout()
-        vbox.addLayout(hbox1)  
-        vbox.addLayout(hbox2)  
-        vbox.addLayout(hbox3)  
-        vbox.addStretch(1)
-
-        self.parent.setLayout(vbox) #This line makes sure that all widgest defined so far are assigned to the widget defines in self.parent
-        
-        self.parent.resize(self.parent.minimumSize())
-
-        self.widgets_enabled_when_connected = [self.button_SetAutoScale, 
-                                               self.combo_TimeConstants , 
-                                               self.label_Sensitivities,
-                                               self.combo_Sensitivities,
-                                               self.button_StartPauseReading,
-                                               self.button_StopReading]
-
-        self.widgets_enabled_when_disconnected = [self.combo_Devices , 
-                                                  self.button_RefreshDeviceList]
-
-        self.click_button_refresh_list_devices()    #By calling this method, as soon as the gui is created we also look for devices
-        self.set_disconnected_state()               #When GUI is created, all widgets are set to the "Disconnected" state
-        return self
-
-    def create_plot(self):
+    def refresh_list_devices(self):
         '''
-        This function creates an additional (separated) window with a pyqtgraph object, which plots the contents of self.stored_powers
-        '''
-        self.plot_window = Qt.QWidget() #This is the widget that will contain the plot. Since it does not have a parent, the plot will be in a floating (separated) window
-        self.plot_object = PlotObject(self.app, self.mainwindow, self.plot_window)
-        styles = {"color": "#fff", "font-size": "20px"}
-        self.plot_object.graphWidget.setLabel("left", "Power", **styles)
-        self.plot_object.graphWidget.setLabel("bottom", "Acqusition #", **styles)
-        self.plot_window.setWindowTitle(f"Powermeter: {self.connected_device_name}")
-        self.plot_window.show()
-        
-### END functions to create GUI
-
-### GUI Events Functions
-
-    def click_button_refresh_list_devices(self):
-        '''
-        Get a list of all device connected to the computer, by using the method ListDevices(). For each device we obtain its identity and its address.
-        For each device, we create the string "identity -->  address" and we add the string to the corresponding combobox
+        Get a list of all devices connected, by using the method list_devices() of the driver. For each device obtain its identity and its address.
+        For each device, create the string "identity -->  address" and add the string to the corresponding combobox in the GUI 
         '''
         #First we empty the combobox
-        self.combo_Devices.clear()
+        self.gui.combo_Devices.clear()
         
         #Then we read the list of devices
         self.logger.info(f"Looking for devices...") 
         list_valid_devices = self.instrument.list_devices()
         if(len(list_valid_devices)>0):
             list_IDNs_and_devices = [dev[1] + " --> " + dev[0] for dev in list_valid_devices] 
-            #list_IDNs_and_devices = [i for i in list_IDN] 
             self.combo_Devices.addItems(list_IDNs_and_devices)  
         self.logger.info(f"Found {len(list_valid_devices)} devices.") 
 
-    def click_button_connect_disconnect(self):
-        ''' 
-        This function establishes the connection to the device selected in the combobox list, or disconnects the device, depending on the value of the variable self.instrument.connected .   
-        If self.instrument.connected == False, then the device is not connected, and we attempt connection
-        If self.instrument.connected == True, then the device is connected, and we attempt disconnection
-        '''
-        
-        if(self.instrument.connected == False): # We attenpt connection       
-            self.connect_device()
-        elif(self.instrument.connected == True): # We attenpt disconnection
-            self.disconnect_device()
-            
-    def click_button_set_auto_scale(self):
-        ID = self.instrument.set_auto_scale()
-        if ID==0:
-            self.logger.error(f"It was not possible to set 'auto scale'. This device model might not support it.")
-        else:
-            self.logger.info(f"'auto scale' set correctly.")
-
-    def click_combo_time_constants(self):
-        self.set_time_constant(self.combo_TimeConstants.currentIndex())
-
-    def click_combo_sensitivities(self):
-        self.set_sensitivity(self.combo_Sensitivities.currentIndex())
-       
-    def click_button_StartPauseReading(self): 
-        if(self.continuous_read == False):
-            self.start_reading()
-        elif (self.continuous_read == True):
-            self.pause_reading()
-        return
-
-    def click_button_StopReading(self):
-        self.stop_reading()
-
-    def press_enter_refresh_time(self):
-        refresh_time = self.edit_RefreshTime.text()
-        try: 
-            refresh_time = float(refresh_time)
-            if self.refresh_time == refresh_time: #in this case the number in the refresh time edit box is the same as the refresh time currently stored
-                return True
-        except ValueError:
-            self.logger.error(f"The refresh time must be a valid number.")
-            self.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
-            return False
-        if refresh_time < 0.001:
-            self.logger.error(f"The refresh time must be positive and >= 1ms.")
-            self.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
-            return False
-        self.logger.info(f"The refresh time is now {refresh_time} s.")
-        self.refresh_time = refresh_time
-        self.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
-        return True
-
-    def click_button_ShowHidePlot(self):
-        self.plot_window.setHidden(not self.plot_window.isHidden())
-        #self.plot_frame.activate()
-        #self.parent.resize(self.parent.minimumSizeHint())
-
-### END GUI Events Functions
-
-### Functions to talk with low-level driver
-
-    def connect_device(self):
-        device_full_name = self.combo_Devices.currentText() # Get the device name from the combobox
+    def connect_device(self,device_full_name):
         if(device_full_name==''): # Check  that the name is not empty
             self.logger.error("No valid device has been selected")
             return
@@ -344,48 +111,18 @@ class interface():
             self.set_disconnected_state()
         else: #If disconnection was not successful
             self.logger.error(f"Error: {Msg}")
- 
-    def disable_widget(self,widgets):
-        for widget in widgets:
-            widget.setEnabled(False)   
 
-    def enable_widget(self,widgets):
-        for widget in widgets:
-            widget.setEnabled(True)   
+    def set_auto_scale(self):
+        ID = self.instrument.set_auto_scale()
+        if ID==0:
+            self.logger.error(f"It was not possible to set 'auto scale'. This device model might not support it.")
+        else:
+            self.logger.info(f"'auto scale' set correctly.")
 
-    def set_disconnected_state(self):
-        self.disable_widget(self.widgets_enabled_when_connected)
-        self.enable_widget(self.widgets_enabled_when_disconnected)
-        self.button_ConnectDevice.setText("Connect")
-
-    def set_connecting_state(self):
-        self.disable_widget(self.widgets_enabled_when_connected)
-        self.enable_widget(self.widgets_enabled_when_disconnected)
-        self.button_ConnectDevice.setText("Connecting...")
-
-    def set_connected_state(self):
-        self.enable_widget(self.widgets_enabled_when_connected)
-        self.disable_widget(self.widgets_enabled_when_disconnected)
-        self.button_ConnectDevice.setText("Disconnect")
-        #self.read_wavelength()
-        #self.read_status_power_autorange()
-        #self.read_power_range()
-        #If a self.plot_object was created, update window title with device name and vertical axis of plot with current Power units
-        if self.plot_object: 
-            #self.plot_object.graphWidget.setLabel("left", f"Power [{self.instrument._power_units}]")
-            self.plot_window.setWindowTitle(f"Powermeter: {self.connected_device_name}")
-
-    def set_pause_state(self):
-        self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'play.png')))
-    def set_reading_state(self):
-        self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'pause.png')))
-    def set_stopped_state(self):    
-        self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'play.png')))
 
     def populate_combo_time_constants(self):
         self.logger.info(f"Reading all possible values of time constants from device...")
         self.time_constants = self.instrument.time_constants
-	
         self.combo_TimeConstants.clear() #First we empty the combobox
         if len(self.time_constants)>0:
                 self.combo_TimeConstants.addItems([str(tc) for tc in self.time_constants])  
@@ -394,7 +131,6 @@ class interface():
     def populate_combo_sensitivities(self):
         self.logger.info(f"Reading all possible values of sensitivities from device...")
         self.sensitivities = self.instrument.sensitivities
-	
         self.combo_Sensitivities.clear() #First we empty the combobox
         if len(self.sensitivities)>0:
                 self.combo_Sensitivities.addItems([str(s) for s in self.sensitivities])  
@@ -414,9 +150,13 @@ class interface():
             return
         self.logger.info(f"Setting time constant to {self.time_constants[tc]}  (index={tc})...")
         try: 
-            self.instrument.time_constant = tc
-            self.time_constant = tc
-            self.logger.info(f"Time constant set correctly.")
+            self.instrument.time_constant = tc #set time constant
+            tcnew = self.instrument.time_constant #read again time constant to confirm that it changed
+            if tcnew == tc:
+                self.logger.info(f"Time constant changed to {self.time_constants[tc]} (old value was {self.time_constants[self.time_constant]}).")
+                self.time_constant = tc
+            else:
+                self.logger.error(f"The driver did not raise any error, but it was not possible to change the time constant.")
         except Exception as e:
             self.logger.error(f"An error occurred while setting the time constant: {e}")
             return False
@@ -436,13 +176,35 @@ class interface():
             return
         self.logger.info(f"Setting sensitivity to {self.sensitivities[s]}  (index={s})...")
         try: 
-            self.instrument.sensitivity = s
-            self.sensitivity = s
-            self.logger.info(f"Sensitivity set correctly.")
+            self.instrument.sensitivity = s #set sensitivity
+            snew = self.instrument.sensitivity #read again sensitivity to confirm that it changed
+            if snew == s:
+                self.logger.info(f"Sensitivity changed to {self.sensitivities[s]} (old value was {self.sensitivities[self.sensitivity]}).")
+                self.sensitivity = s
+            else:
+                self.logger.error(f"The driver did not raise any error, but it was not possible to change the sensitivity.")
         except Exception as e:
             self.logger.error(f"An error occurred while setting the sensitivity: {e}")
             return False
         return True
+
+    def set_refresh_time(self, refresh_time):
+        try: 
+            refresh_time = float(refresh_time)
+            if self.refresh_time == refresh_time: #in this case the number in the refresh time edit box is the same as the refresh time currently stored
+                return True
+        except ValueError:
+            self.logger.error(f"The refresh time must be a valid number.")
+            self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+            return False
+        if refresh_time < 0.001:
+            self.logger.error(f"The refresh time must be positive and >= 1ms.")
+            self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+            return False
+        self.logger.info(f"The refresh time is now {refresh_time} s.")
+        self.refresh_time = refresh_time
+        self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+        return True
    
     def start_reading(self):
         if(self.instrument.connected == False):
@@ -494,15 +256,9 @@ class interface():
             self.output['theta'] = theta
             for k in self.output.keys():	
                 self.stored_data[k].append(self.output[k])
-            if hasattr(self,'trigger'):
-                QtCore.QTimer.singleShot(int(self.trigger[1]), self.trigger[0]) 
 
-            #StoredPowers_np = np.array(self.StoredPowers)
-            #Fluctuation = 100 * StoredPowers_np[-10:].ptp(axis=0)  / StoredPowers_np[-10:].mean(axis=0)  
-            
-            #self.PowerFluctuationString.set("{:.2f}".format(Fluctuation) + " %")
+            super().update()
 
-            #super().Update()
             for k in self.output.keys():
                 try:
                     self.edit_Output[k].setText(f"{self.output[k]:.2e}")
@@ -512,23 +268,205 @@ class interface():
            
         return
 
-    def set_trigger(self,external_function,delay=0):
-        '''
-        This method allows to use this device as a trigger for other operations. Every time that this interface object acquires data from the device (i.e. every time 
-        the function self.update is executed with self.continuous_read == True), the function external_function is also called. external_function must be a valid function which does not 
-        require any input parameter.
-        The optional parameter delay sets a delay (in ms) between the call to the function update and the call to the function external_function
-        '''
-        if not(inspect.isfunction(external_function)):
-            self.logger.error(f"Input parameter external_function must be a valid function")    
-        self.logger.info(f"Creating a trigger for this device...")
-        self.trigger = [external_function, delay]
+class gui(abstract_gui):
+    def __init__(self,interface,parent,plot=False):
+        super().__init__(interface,parent)
 
-    def close(self):
-        if (self.instrument.connected == True):
-            self.disconnect_device()
+        self.widgets_enabled_when_connected = []     #The widgets in this list will only be enabled when the interface has succesfully connected to a powermeter
+        self.widgets_enabled_when_disconnected = []  #The widgets in this list will only be enabled when the interface is not connected to a powermeter
+        
+        hbox1 = Qt.QHBoxLayout()
+        self.label_DeviceList = Qt.QLabel("Devices: ")
+        self.combo_Devices = Qt.QComboBox()
+        #self.combo_Devices.setFixedWidth(300)
+        self.button_RefreshDeviceList = Qt.QPushButton("")
+        self.button_RefreshDeviceList.setIcon(QtGui.QIcon(path.join(graphics_dir,'refresh.png')))
+        self.button_RefreshDeviceList.clicked.connect(self.click_button_refresh_list_devices)
+        hbox1.addWidget(self.label_DeviceList)
+        hbox1.addWidget(self.combo_Devices,stretch=1)
+        hbox1.addWidget(self.button_RefreshDeviceList)
+        #hbox1.addStretch(1)
+
+        hbox2 = Qt.QHBoxLayout()
+        self.button_ConnectDevice = Qt.QPushButton("Connect")
+        self.button_ConnectDevice.clicked.connect(self.click_button_connect_disconnect)
+        self.button_SetAutoScale = Qt.QPushButton("Set Auto Scale")
+        self.button_SetAutoScale.clicked.connect(self.click_button_set_auto_scale)
+        self.label_TimeConstants = Qt.QLabel("Time constant (s): ")
+        self.combo_TimeConstants = Qt.QComboBox()
+        self.combo_TimeConstants.activated.connect(self.click_combo_time_constants)
+
+        self.label_Sensitivities = Qt.QLabel("Sensitivity (V): ")
+        self.combo_Sensitivities = Qt.QComboBox()
+        self.combo_Sensitivities.activated.connect(self.click_combo_sensitivities)
+
+        hbox2.addWidget(self.button_ConnectDevice)
+        hbox2.addWidget(self.button_SetAutoScale)
+        hbox2.addWidget(self.label_TimeConstants)
+        hbox2.addWidget(self.combo_TimeConstants,stretch=1)
+        hbox2.addWidget(self.label_Sensitivities)
+        hbox2.addWidget(self.combo_Sensitivities,stretch=1)
+        #hbox2.addStretch(1)
+
+        hbox3 = Qt.QHBoxLayout()
+        hbox3_vbox1 = Qt.QVBoxLayout()
+        
+        self.button_StartPauseReading = Qt.QPushButton("")
+        self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'play.png')))
+        self.button_StartPauseReading.setToolTip('Start or pause the reading from the device. The previous data points are not discarded when pausing.') 
+        self.button_StartPauseReading.clicked.connect(self.click_button_StartPauseReading)
+        if plot:
+            self.button_StopReading = Qt.QPushButton("")
+            self.button_StopReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'stop.png')))
+            self.button_StopReading.setToolTip('Stop the reading from the device. All previous data points are discarded.') 
+            self.button_StopReading.clicked.connect(self.click_button_StopReading)
+        else:
+            self.button_StopReading = None
+        hbox3_vbox1_hbox1 = Qt.QHBoxLayout()
+        hbox3_vbox1_hbox1.addWidget(self.button_StartPauseReading)  
+        if plot:
+            hbox3_vbox1_hbox1.addWidget(self.button_StopReading)
+        
+        
+        self.label_RefreshTime = Qt.QLabel("Refresh time (s): ")
+        self.label_RefreshTime.setToolTip('Specifies how often data is read from the device (Minimum value = 0.001 s).') 
+        self.edit_RefreshTime  = Qt.QLineEdit()
+        self.edit_RefreshTime.setText(f"{self.interface.refresh_time:.3f}")
+        self.edit_RefreshTime.setToolTip('Specifies how often data is read from the device (Minimum value = 0.001 s).') 
+        self.edit_RefreshTime.returnPressed.connect(self.press_enter_refresh_time)
+        self.edit_RefreshTime.setAlignment(QtCore.Qt.AlignRight)
+        #self.edit_RefreshTime.setMaximumWidth(50)
+        hbox3_vbox1_hbox2 = Qt.QHBoxLayout()
+        hbox3_vbox1_hbox2.addWidget(self.label_RefreshTime)  
+        hbox3_vbox1_hbox2.addWidget(self.edit_RefreshTime,stretch=1)
+
+        hbox3_vbox1.addLayout(hbox3_vbox1_hbox1)
+        hbox3_vbox1.addLayout(hbox3_vbox1_hbox2)
+
+
+        fontLabel = QtGui.QFont("Times", 10,QtGui.QFont.Bold)
+        fontEdit = QtGui.QFont("Times", 10,QtGui.QFont.Bold)
+
+        self.label_Output = dict()    
+        self.edit_Output = dict() 
+        self.widget_Output = dict()     
+        for k in self.interface.output.keys():		
+            self.label_Output[k] = Qt.QLabel(f"{k}: ")
+            self.label_Output[k].setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignCenter)
+            self.label_Output[k].setFont(fontLabel)
+            self.edit_Output[k] = Qt.QLineEdit()
+            self.edit_Output[k].setFont(fontEdit)
+            self.edit_Output[k].setMinimumWidth(60)
+            self.edit_Output[k].setAlignment(QtCore.Qt.AlignRight)
+            self.edit_Output[k].setReadOnly(True)
+            hbox_temp = Qt.QHBoxLayout()
+            hbox_temp.addWidget(self.label_Output[k])
+            hbox_temp.addWidget(self.edit_Output[k],stretch=1)
+            self.widget_Output[k] = Qt.QWidget()
+            self.widget_Output[k].setLayout(hbox_temp)
+
+        self.layout_ContainerOutputs = Qt.QGridLayout()
+        self.layout_ContainerOutputs.addWidget(self.widget_Output['X'], 0, 0)
+        self.layout_ContainerOutputs.addWidget(self.widget_Output['Y'], 1, 0)
+        self.layout_ContainerOutputs.addWidget(self.widget_Output['mag'], 0, 1)
+        self.layout_ContainerOutputs.addWidget(self.widget_Output['theta'], 1, 1)
+        self.layout_ContainerOutputs.setSpacing(0)
+        self.layout_ContainerOutputs.setContentsMargins(0, 0, 0, 0)
+        self.widget_ContainerOutputs = Qt.QWidget()
+        self.widget_ContainerOutputs.setLayout(self.layout_ContainerOutputs)
+        #self.button_ShowHidePlot = Qt.QPushButton("Show/Hide Plot")
+        #self.button_StopReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'stop.png')))
+        #self.button_ShowHidePlot.setToolTip('Show/Hide Plot.') 
+        #self.button_ShowHidePlot.clicked.connect(self.click_button_ShowHidePlot)
+
+        hbox3.addLayout(hbox3_vbox1)
+        #hbox3.addStretch(1)
+        hbox3.addWidget(self.widget_ContainerOutputs,stretch=1)
+        #hbox3.addWidget(self.button_ShowHidePlot)
+        #hbox3.addStretch(1)
+        #hbox3.setAlignment(QtCore.Qt.AlignTop)
+                
+        self.container = Qt.QVBoxLayout()
+        self.container.addLayout(hbox1)  
+        self.container.addLayout(hbox2)  
+        self.container.addLayout(hbox3)  
+        self.container.addStretch(1)
+
+        self.widgets_enabled_when_connected = [self.button_SetAutoScale, 
+                                               self.combo_TimeConstants , 
+                                               self.label_Sensitivities,
+                                               self.combo_Sensitivities,
+                                               self.button_StartPauseReading,
+                                               self.button_StopReading]
+
+        self.widgets_enabled_when_disconnected = [self.combo_Devices , 
+                                                  self.button_RefreshDeviceList]
     
+    def initialize(self):
+        super().initialize()
+        self.click_button_refresh_list_devices()    #By calling this method, as soon as the gui is created we also look for devices
+        self.set_disconnected_state()               #When GUI is created, all widgets are set to the "Disconnected" state
 
+        
+    def set_disconnected_state(self):
+        self.disable_widget(self.widgets_enabled_when_connected)
+        self.enable_widget(self.widgets_enabled_when_disconnected)
+        self.button_ConnectDevice.setText("Connect")
+
+    def set_connecting_state(self):
+        self.disable_widget(self.widgets_enabled_when_connected)
+        self.enable_widget(self.widgets_enabled_when_disconnected)
+        self.button_ConnectDevice.setText("Connecting...")
+
+    def set_connected_state(self):
+        self.enable_widget(self.widgets_enabled_when_connected)
+        self.disable_widget(self.widgets_enabled_when_disconnected)
+        self.button_ConnectDevice.setText("Disconnect")
+
+    def set_pause_state(self):
+        self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'play.png')))
+    def set_reading_state(self):
+        self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'pause.png')))
+    def set_stopped_state(self):    
+        self.button_StartPauseReading.setIcon(QtGui.QIcon(path.join(graphics_dir,'play.png')))
+
+    ### GUI Events Functions
+
+    def click_button_refresh_list_devices(self):
+        self.interface.refresh_list_devices()
+
+    def click_button_connect_disconnect(self):
+        if(self.interface.instrument.connected == False): # We attempt connection   
+            device_full_name = self.combo_Devices.currentText() # Get the device name from the combobox
+            self.interface.connect_device(device_full_name)
+        elif(self.interface.instrument.connected == True): # We attempt disconnection
+            self.interface.disconnect_device()
+            
+    def click_button_set_auto_scale(self):
+        self.interface.set_auto_scale()
+
+    def click_combo_time_constants(self):
+        self.set_time_constant(self.combo_TimeConstants.currentIndex())
+
+    def click_combo_sensitivities(self):
+        self.set_sensitivity(self.combo_Sensitivities.currentIndex())
+       
+    def click_button_StartPauseReading(self): 
+        if(self.continuous_read == False):
+            self.start_reading()
+        elif (self.continuous_read == True):
+            self.pause_reading()
+        return
+
+    def click_button_StopReading(self):
+        self.stop_reading()
+
+    def press_enter_refresh_time(self):
+        return self.interface.set_refresh_time(self.edit_RefreshTime.text())
+
+### END GUI Events Functions
+
+ 
 class MainWindow(Qt.QWidget):
     def __init__(self):
         super().__init__()
@@ -546,11 +484,11 @@ def main():
 
     app = Qt.QApplication(sys.argv)
     window = MainWindow()
-    Interface = interface(app,window,window) #In this case window is both the MainWindow and the parent of the gui
+    Interface = interface(app=app,mainwindow=window) #In this case window is both the MainWindow and the parent of the gui
     Interface.verbose = not(args.decrease_verbose)
     app.aboutToQuit.connect(Interface.close) 
-    Interface.create_gui(plot=True)
-    Interface.set_trigger(lambda : print('test'),delay=1000)
+    Interface.create_gui(window,plot=False)
+    #Interface.set_trigger(lambda : print('test'),delay=1000)
     window.show()
     app.exec()# Start the event loop.
 
